from setuptools import find_packages, setup

setup(
    name='yev_lib',
    packages=find_packages(),
    version='0.1.0',
    description='A short description of the project.',
    author='Yevhenii Maslov',
    license='',
)
